/*  -*- pse-c -*-
 *----------------------------------------------------------------------------
 * Filename: agp_test.c
 * $Revision: 1.5 $
 *----------------------------------------------------------------------------
 * Unit level test for IEGD AGP
 * Copyright © 2009 Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

/* Copyright 2003 - 2005 Intel Corporation. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *	Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *
 *      Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *      Neither the name Intel Corporation nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <stdio.h>
#include <linux/agpgart.h>
#include <sys/mman.h>

/*#define PAGE_SIZE 1024*/

#define VERBOSE "-v"

int verbose = 0;
int file_desc, temp, length;
unsigned char *mmap_gart;

int init_agp(void)
{
	agp_info info;
	agp_setup setup;
	
	if (verbose)
	{
		printf("Testing ioctl AGPIOC_ACQUIRE.\n");	
	}
	if(ioctl(file_desc, AGPIOC_ACQUIRE) != 0)
	{
		printf("Error on AGPIOC_ACQUIRE.\n");
		printf("Reinstall IKM.\n");
		exit(-1);
	}
	if (verbose)
	{
		printf("Testing ioctl call for info init\n");
	}
	if(ioctl(file_desc, AGPIOC_INFO, &info) != 0)
	{
		printf("Error on AGPIOC_INFO\n");
		printf("Reinstall IKM.\n");
		exit(-1);
	}
	if (verbose)
	{
		printf("Testing init info\n version:%i.%i,\n id:0x%lx,\n mode:0x%lx,\n" 
		 " base:0x%lx,\n size:%i,\n total mem:%i,\n system mem:%i,\n"
		 " used mem:%i\n", info.version.major, info.version.minor, 
		 info.bridge_id, info.agp_mode, info.aper_base, info.aper_size, 
		 info.pg_total, info.pg_system, info.pg_used);
	
		printf("Testing mmap the device\n");
	}
	length = info.aper_size*0x100000;
	mmap_gart = mmap(NULL, info.aper_size*0x100000, 
	 PROT_READ | PROT_WRITE, MAP_SHARED, file_desc, 0);
	if(mmap_gart == (unsigned char *) 0xFFFFFFFF)
	{
		printf("Error on mmap\n");
		printf("Reinstall IKM.\n");
		close(file_desc);
		exit(-1);
	}	

	setup.agp_mode = info.agp_mode;
	if (verbose)
	{
		printf("Testing ioctl AGPIOC_SETUP\n");
	}
	if(ioctl(file_desc, AGPIOC_SETUP, &setup) != 0)
	{
		printf("Error on AGPIOC_SETUP\n");
		printf("Reinstall IKM.\n");
		exit(-1);
	}
}

void gart_unbind(int key)
{
	agp_unbind unbind;
	unbind.key = key;
	unbind.priority = 0;
	if (verbose)
	{
		printf("Testing ioctl AGPIOC_UNBIND\n");
	}
	if(ioctl(file_desc, AGPIOC_UNBIND, &unbind) != 0)
	{
		printf("Error on AGPIOC_UNBIND\n");
		printf("Reinstall IKM.\n");
		exit(-1);
	}
}

void gart_bind(int key)
{
	agp_bind bind;	
	bind.key = key;
	agp_info info;

	int page_size = 4000;
	int aper_size, gtt_entries, bind_error;

	if(ioctl(file_desc, AGPIOC_INFO, &info) != 0)
	{
		printf("Error on AGPIOC_INFO\n");
		printf("Reinstall IKM.\n");
		exit(-1);
	}
	aper_size = info.aper_size;
	gtt_entries = aper_size*1000000/page_size;
	if (verbose)
	{
		printf("max memory: %i\n", gtt_entries);
	}

	do
	{
		bind_error = 0;		
		bind.pg_start = page_size;
		printf("Testing ioctl AGPIOC_BIND\n");
		if(ioctl(file_desc, AGPIOC_BIND, &bind) != 0)
		{
			if (page_size < gtt_entries)
			{
				page_size = page_size+4000;
				printf("Trying new address for bind %i.\n", page_size);
				bind_error = 1;
			}
			else
			{
				printf("Error on AGPIOC_BIND\n");
				printf("Reinstall IKM.");
				exit(-1);
			}
		}
	} while (bind_error);
	printf("Sucessful bind.\n");
}

int gart_alloc(int count)
{
	agp_allocate allocate;
	
	allocate.type = 0;
	allocate.pg_count = count;
	if (verbose)
	{
		printf("Testing ioctl AGPIOC_ALLOCATE\n");
	}
	if(ioctl(file_desc, AGPIOC_ALLOCATE, &allocate) != 0)
	{
		printf("Error on AGPIOC_ALLOCATE\n");
		printf("Reinstall IKM.");
		exit(-1);
	}

	gart_bind(allocate.key);
	
	return(allocate.key);
}

void gart_free(int key)
{
	
	gart_unbind(key);	
	if (verbose)
	{	
		printf("Testing ioctl AGPIOC_DEALLOCATE\n");	
	}
	if(ioctl(file_desc, AGPIOC_DEALLOCATE, key) != 0)
	{
		printf("Error on AGPIOC_DEALLOCATE\n");
		printf("Reinstall IKM.\n");
		exit(-1);
	}
}

int main(int argc, char *argv[])
{
	/* Check for verbose mode */	
	int i, key, key1;
	agp_info info;	
	
	for (i = 1; i < argc; i++)
	{
		if(strcmp(argv[1], VERBOSE) == 0)
		{
			verbose = 1;
			printf("Verbose mode.\n");
		}
	}	
	
	/* Open the agpgart */
	file_desc=open("/dev/agpgart",O_RDWR);

	if(file_desc<0){
		printf("Cannot open device file:%d\n",file_desc);
		printf("Check for root level permissions.");
		exit(-1);
	}
	
	if (verbose)
	{
		printf("Open device file:%d\n",file_desc);
		/* This the ioctl that allocates physical memory */
		printf("Testing ioctl for memory allocation\n");
	}

	init_agp();

	key = gart_alloc(64);
	key1 = gart_alloc(0);
	if (verbose)
	{
		printf("Testing ioctl call for info\n");
	}
	if(ioctl(file_desc, AGPIOC_INFO, &info) != 0)
	{
		close(file_desc);		
		printf("Error on AGPIOC_INFO\n");
		printf("Reinstall IKM.\n");
		exit(-1);
	}
	
	if (verbose)
	{
		printf("Testing init info\n version:%i.%i,\n id:0x%lx,\n mode:0x%lx,\n" 
		 " base:0x%lx,\n size:%i,\n total mem:%i,\n system mem:%i,\n"
		 " used mem:%i\n", info.version.major, info.version.minor, 
		 info.bridge_id, info.agp_mode, info.aper_base, info.aper_size, 
		 info.pg_total, info.pg_system, info.pg_used);
	}
	
	gart_free(key);
	gart_free(key1);

	if (munmap(mmap_gart, length) < 0)
	{
		close(file_desc);		
		printf("Error on munmap\n");
		printf("Reinstall IKM.\n");
		exit(-1);
	}
	
	if (verbose)
	{
		printf("Testing ioctl AGPIOC_RELEASE\n");
	}
	if(ioctl(file_desc, AGPIOC_RELEASE) != 0)
	{
		close(file_desc);		
		printf("Error on AGPIOC_RELEASE\n");
		printf("Reinstall IKM.");
		exit(-1);
	}

	printf("AGPGART successfully loaded\n");
	
	close(file_desc);
	
	return 0;
	

}
