/*  -*- pse-c -*-
 *----------------------------------------------------------------------------
 * Filename: drm_test.c
 * $Revision: 1.3 $
 *----------------------------------------------------------------------------
 * Unit level test for IEGD DRM
 * Copyright © 2009 Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

/* Copyright 2003 - 2005 Intel Corporation. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *	Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *
 *      Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *      Neither the name Intel Corporation nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "iegd.h"
#include "iegd_drm_client.h"

#define DRM_IOCTL_BASE			'd'
#define DRM_IO(nr)			_IO(DRM_IOCTL_BASE,nr)
#define DRM_IOR(nr,type)		_IOR(DRM_IOCTL_BASE,nr,type)
#define DRM_IOW(nr,type)		_IOW(DRM_IOCTL_BASE,nr,type)
#define DRM_IOWR(nr,type)		_IOWR(DRM_IOCTL_BASE,nr,type)

#define DRM_IOCTL_INTEL_GETPAGES         DRM_IOWR(DRM_BASE_COMMAND + \
				DRM_INTEL_GETPAGES, drm_intel_getpages_t)
#define DRM_IOCTL_INTEL_FREEPAGES         DRM_IOWR(DRM_BASE_COMMAND + \
				DRM_INTEL_FREEPAGES, drm_intel_freepages_t)
#define DRM_IOCTL_INTEL_INFO_INIT		DRM_IOW( DRM_BASE_COMMAND + \
				DRM_INTEL_INFO_INIT, intel_drm_info_t)
#define DRM_IOCTL_INTEL_INFO_GET		DRM_IOR( DRM_BASE_COMMAND + \
				DRM_INTEL_INFO_GET, intel_drm_info_t)

#define PAGE_SIZE 4096

#define VERBOSE "-v"

int main(int argc, char *argv[])
{
	int file_desc, ret_value, i;
	unsigned long *virt_ptr;
	int failed=0;

	/* Check for verbose mode */	
	int index;	
	int verbose = 0;
	
	for (index = 1; index < argc; index++)
	{
		if(strcmp(argv[index], VERBOSE) == 0)
		{
			verbose = 1;
			printf("Verbose mode.\n");
		}
	}

	if (verbose)
	{
		printf("Starting client\n");
	}
	/* Open the drm */
	file_desc=open("/dev/dri/card0",O_RDWR);

	if(file_desc<0){
	/* In case of a different /dev tree struct.
	 * try /dev/card0
	 */
		file_desc=open("/dev/card0",O_RDWR);
	}

	if(file_desc<0){
		printf("Can't open device file:%s\n",DRIVER_DESC);
		printf("Check for root level permissions.");
		printf("Reinstall IKM.\n");
		exit(-1);
	}
	
	if (verbose)
	{
		printf("Open device file:%d\n",file_desc);
		/* This the ioctl that allocates physical memory */
		printf("Testing ioctl for memory allocation\n");
	}
	
	drm_intel_getpages_t getpages;
	/* set the number of bytes we want the drm to allocate */
	getpages.size=(PAGE_SIZE- 1000); 

	ret_value=ioctl(file_desc,DRM_IOCTL_INTEL_GETPAGES,&getpages);
	if (verbose)
	{
		printf("IOCTL call memory allocation test:");
	}
	if(ret_value<0){
		printf("DRM module failed memory allocation test.\n");
		printf("Reinstall IKM.\n");
		exit(-1);
	}
	if (verbose)
	{
		printf(" Success\n");
		printf("size: %d,phy_address: %#x,virt_address: %#x,offset: %#x\n",
	 	 getpages.size, getpages.phy_address, getpages.virt_address, 
	 	 getpages.offset);

		/* test for memory access */

		printf("Testing ioctl for memory access\n");
	}
	
	virt_ptr=(unsigned long *)getpages.virt_address;

	/* input 0..10 into subsequent memory */

	for(i=0;i<=11;i++){
	*virt_ptr=i;
	virt_ptr++;
	}

	/*read from subsequent memory */
	virt_ptr=(unsigned long *)getpages.virt_address;
	for(i=0;i<=11;i++){
		if (verbose)
		{
			printf("virt_ptr @ %#x,value: %d\n",virt_ptr,*virt_ptr);
		}
		if(*virt_ptr!=i){
			printf("Failed memory read.\n");
		}
	virt_ptr++;
	}
	if (verbose)
	{
		printf("IOCTL call memory access test:");
	}
	if(failed){
		printf("DRM module failed memory access test.\n");
		printf("Reinstall IKM.\n");
		exit(-1);
	}
	if (verbose)
	{
		printf(" Success\n");
	}
	/* freeing memory */

	drm_intel_freepages_t freepages;
	freepages.size=getpages.size;
	freepages.phy_address=getpages.phy_address;
	freepages.virt_address=getpages.virt_address;
	if (verbose)
	{
		printf("Freeing phy_address:%#x,size:%#x\n",
	 	 freepages.phy_address,freepages.size);

		printf("Testing ioctl call for info init\n");
	}
	/* init the drm info structure in the drm and test its value */
	intel_drm_info_t info;
	intel_drm_info_t test_info;
	info.device_id=0x456;
	info.revision=333;
	info.video_memory_offset=0x10245;
	info.video_memory_size=987;
	info.hw_status_offset=0x444;
	if (verbose)
	{

		printf("Testing init info device_id: %#x,revision: %d,offset: %#x," 
	 	 " size: %d, hw_status_offset: %lx\n", info.device_id, info.revision, 
	 	 info.video_memory_offset, info.video_memory_size, 
	 	 info.hw_status_offset);
	}
	 
	ioctl(file_desc,DRM_IOCTL_INTEL_INFO_INIT,&info);

	if (verbose)
	{
		printf("Alternative data to init\n");
	}
	/* init the drm info structure in the drm and test its value */
	info.device_id=0x123;
	info.revision=456;
	info.video_memory_offset=0x789;
	info.video_memory_size=111;
	info.hw_status_offset=0x555;
	
	if (verbose)
	{
		printf("Testing init 2nd info device_id: %#x,revision: %d,offset: %#x,"
	 	 " size: %d, hw_status_offset: %lx\n", info.device_id, info.revision, 
	 	 info.video_memory_offset, info.video_memory_size, 
	 	 info.hw_status_offset);

		printf("Get init info\n");
	}
	ioctl(file_desc,DRM_IOCTL_INTEL_INFO_GET,&test_info);
	
	if (verbose)
	{
		printf("IOCTL call for info init:");
		printf("Got init info device_id: %#x,revision: %d," 
	 	 "offset: %#x,size:%d, hw_status_offset: %lx\n", test_info.device_id, 
	 	 test_info.revision, test_info.video_memory_offset, 
	 	 test_info.video_memory_size, test_info.hw_status_offset);
	}

	/* compare with original data to see if its still the same */
	info.device_id=0x456;
	info.revision=333;
	info.video_memory_offset=0x10245;
	info.video_memory_size=987;
	info.hw_status_offset=0x444;
	failed=0;

	if(info.device_id!=test_info.device_id){
		failed=1;
	}
	if(info.revision!=test_info.revision){
		failed=1;
	} 
	if(info.video_memory_offset!=test_info.video_memory_offset){
		failed=1;
	}
	if(info.video_memory_size!=test_info.video_memory_size){
		failed=1;
	}
	if(info.hw_status_offset!=test_info.hw_status_offset){
		failed=1;
	}
	
	if(failed){
		printf("DRM module failed IOCTL info did not match.\n");
		printf("Reinstall IKM.");
		exit(-1);
	}

	printf("DRM successfully loaded\n");

	close(file_desc);
	
	return 0;

}
