/*  -*- pse-c -*-
 *----------------------------------------------------------------------------
 * Filename: iegd_interface.c
 * $Revision: 1.15 $
 *----------------------------------------------------------------------------
 * <>
 * Copyright  2008, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *
 *----------------------------------------------------------------------------
 * Functions:
 *
 *
 *----------------------------------------------------------------------------
 */

#ifndef _KERNEL_ABS_LAYER
#define _KERNEL_ABS_LAYER

#include <linux/version.h>

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21)
#define IGD_FREE_MEM(a) agp_free_page_array(a)
#else
#define IGD_FREE_MEM(a) vfree((a)->memory)
#endif

#if LINUX_VERSION_CODE == KERNEL_VERSION(2,6,5)
#define DRM_INIT_MODULE() drm_init()
#define DRM_EXIT_MODULE() drm_cleanup()
#define AGP_RET(a) ((a)>=0) ? 1 : 0
#else
#define DRM_INIT_MODULE() intel_init()
#define DRM_EXIT_MODULE() intel_exit()
#define AGP_RET(a) ((a)==0) ? 1 : 0
#endif

#if LINUX_VERSION_CODE<KERNEL_VERSION(2,6,10)
#define IGD_PCI_SAVE_STATE(a,b) pci_save_state(a,b)
#define IGD_PCI_RESTORE_STATE(a,b) pci_restore_state(a,b)
#define pm_message_t u32
#define IGD_IS_SUSPEND(state) ((state)==3)
#else
#define IGD_PCI_SAVE_STATE(a,b) pci_save_state(a)
#define IGD_PCI_RESTORE_STATE(a,b) pci_restore_state(a)
#define IGD_IS_SUSPEND(state) \
	(((state.event)==PM_EVENT_SUSPEND) | ((state.event)==PM_EVENT_FREEZE))
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,23)
#define AGP_LOCK_PAGE(a)
#define AGP_UNLOCK_PAGE(a)
#else
#define AGP_LOCK_PAGE(a) SetPageLocked((a))
#define AGP_UNLOCK_PAGE(a) unlock_page((a))
#endif

#define MASK_PTE(a,b) (a)->driver->masks[(b)].mask
#define AGP_MASK_ADDR(x) MASK_PTE((x),type)

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,25)
#ifndef TRUE
#define TRUE	1
#endif
#ifndef FALSE
#define FALSE	0
#endif
#define SET_PAGES_UC(a,b)	set_pages_uc(a,b)
#define SET_PAGES_WB(a,b)	set_pages_wb(a,b)
#define GLOBAL_FLUSH_TLB()
#ifndef SetPageLocked
#define SetPageLocked(page) set_bit(PG_locked, &page->flags);
#endif
#else
#define SET_PAGES_UC(a,b)	change_page_attr(a,b,PAGE_KERNEL_NOCACHE)
#define SET_PAGES_WB(a,b)	change_page_attr(a,b,PAGE_KERNEL)
#define GLOBAL_FLUSH_TLB()	global_flush_tlb()
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27)
#define ON_EACH_CPU(a,b,c,d)	on_each_cpu(a,b,d)
/* Note: drm_dev_to_irq appears 2.6.28, but some distros
 * have pulled it into earlier versions of their kernel.
 * That's why it's defined here.
 */
#define DRM_DEV_TO_IRQ(a)	drm_dev_to_irq(a)
#else
#define ON_EACH_CPU(a,b,c,d) 	on_each_cpu(a,b,c,d)
#define DRM_DEV_TO_IRQ(a)	(a->irq)
#endif


#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,31)
#define AGP_MEM_TYPE struct page*
#define CONVERT_PAGE_TO_GART(a)  phys_to_gart(page_to_phys(a));
#define AGP_MEMORY_MEMBER pages
#define PAGE_ADDRESS(a) a
#else
#define AGP_MEM_TYPE void*
#define CONVERT_PAGE_TO_GART(a)  a 
#define AGP_MEMORY_MEMBER memory
#define PAGE_ADDRESS(a)          page_address(a)
#endif


/*
 * Kernel interface abstraction. This macro will
 * point to the proper definition for that particular
 * kernel
 */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,31)
#define AGP_MASK_MEMORY(f)   _MASK_MEMORY_PAGE(f)
#else
#define AGP_MASK_MEMORY(f)   _MASK_MEMORY(f)
#endif
#define AGP_CREATE_GATT(f)   _CREATE_GATT_TABLE(f)
#define AGP_FREE_GATT(f)     _FREE_GATT_TABLE(f)
#define AGP_ALLOC_PAGE(f)    _ALLOC_PAGE_AGP(f)
#define AGP_ENABLE(f)        _ENABLE_AGP(f)
#define AGP_TYPE_TO_MASK_TYPE(f) _TYPE_TO_MASK_TYPE(f)

#define AGP_MASK_GTT()   _mask_gtt()
#endif
