#!/bin/bash

#----------------------------------------------------------------------------
# <>
# Copyright © 2009, Intel Corporation.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms and conditions of the GNU General Public License,
# version 2, as published by the Free Software Foundation.
#
# This program is distributed in the hope it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 
# 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
#
#
#----------------------------------------------------------------------------

#
# IKM Kernel Checker Install Script
#

MAKEFILE="Makefile"
KERNELDIR=${KERNELDIR:-/lib/modules/$(uname -r)}
VERSION=/include/linux/version.h
OUTPUT_FILE=".output.txt"
OUTPUT="build.log"
ERRORLOG_FILE="error.log"

if [ -f ${KERNELDIR}${VERSION} ] ; then
	BUILDDIR=${KERNELDIR}
elif [ -f ${KERNELDIR}/build${VERSION} ]; then
	BUILDDIR=${KERNELDIR}/build
fi

VERCODE=`cat ${BUILDDIR}${VERSION} | grep LINUX_VERSION_CODE | awk '{print $3}'`

if [ -e $ERRORLOG_FILE ]; then
	rm -f $ERRORLOG_FILE
fi

if [ -e $OUTPUT ]; then
	rm -f $OUTPUT
fi

# Function to create the makefile
# CFLAGS is not supported for kernel 2.6.27, need to use EXTRA_CFLAGS instead
create_make_file()
{
	if [[ $VERCODE -le 132631 ]]; then
cat > $MAKEFILE << 'IKMKERNELMAKEFILE'
ifneq ($(KERNELRELEASE),)
IKMKERNELMAKEFILE
variable=$(cat <<SETVAR
$1
SETVAR)
echo -e "\t obj-m += $variable" >> $MAKEFILE
cat >> $MAKEFILE << "STOP"
	CFLAGS += -Wno-uninitialized -Wno-unused-variable  

else
	KERNELDIR ?= /lib/modules/$(shell uname -r)/build
	PWD := $(shell pwd)
	
all: clean modules

modules:
	$(MAKE) -C $(KERNELDIR) M=$(PWD) modules
	
clean:
	$(MAKE) -C $(KERNELDIR) M=$(PWD) clean

endif
STOP
else
cat > $MAKEFILE << 'IKMKERNELMAKEFILE'
ifneq ($(KERNELRELEASE),)
IKMKERNELMAKEFILE
variable=$(cat <<SETVAR
$1
SETVAR)
echo -e "\t obj-m += $variable" >> $MAKEFILE
cat >> $MAKEFILE << "STOP"
	EXTRA_CFLAGS += -Wno-uninitialized -Wno-unused-variable  
	
else
	KERNELDIR ?= /lib/modules/$(shell uname -r)/build
	PWD := $(shell pwd)
	
all: clean modules

modules:
	$(MAKE) -C $(KERNELDIR) M=$(PWD) modules
	
clean:
	$(MAKE) -C $(KERNELDIR) M=$(PWD) clean

endif
STOP
fi
}

# Call routine to create makefile
create_make_file pciTest1.o

if [ ! -f "$MAKEFILE" ] ;then
	echo "Makefile creation error"
	exit 1
else
	OUTPUT_FILE=".output.txt"
	make -f $MAKEFILE &> $OUTPUT_FILE
	make -f $MAKEFILE >> $OUTPUT 2>&1
	if [ ! -f pciTest1.o ];then
		echo "Please check the following PCI related functions. Some of the APIs are different." | tee $ERRORLOG_FILE
		echo "1.pci_dev_put" | tee -a $ERRORLOG_FILE
		echo "2.pci_enable_device" | tee -a $ERRORLOG_FILE
		echo "3.pci_find_capability" | tee -a $ERRORLOG_FILE
		echo "4.pci_get_device" | tee -a $ERRORLOG_FILE
		echo "5.pci_register_driver" | tee -a $ERRORLOG_FILE	
		echo "6.pci_unregister_driver " | tee -a $ERRORLOG_FILE
        echo "  " | tee -a $ERRORLOG_FILE
		if [ -f "$OUTPUT_FILE" ];then
			awk '/error/ { for (i=2; i<=NF; i++) printf "%s ", $i; printf "\n"; }' $OUTPUT_FILE | tee -a $ERRORLOG_FILE
		fi
	fi
fi
create_make_file pciTest2.o

if [ ! -f "$MAKEFILE" ] ;then
	echo "Makefile creation error"
	exit 1
else
	OUTPUT_FILE=".output.txt"
	make -f $MAKEFILE &> $OUTPUT_FILE
	make -f $MAKEFILE >> $OUTPUT 2>&1
	
	if [ ! -f pciTest2.o ];then
		echo " " | tee -a $ERRORLOG_FILE
		echo "Please check the following PCI related functions. Some of the APIs are different." | tee -a $ERRORLOG_FILE
		echo "1.pci_read_config_byte" | tee -a $ERRORLOG_FILE
		echo "2.pci_read_config_dword" | tee -a $ERRORLOG_FILE
		echo "3.pci_read_config_word" | tee -a $ERRORLOG_FILE
		echo "4.pci_restore_state" | tee -a $ERRORLOG_FILE
		echo "5.pci_save_state"	 | tee -a $ERRORLOG_FILE
		echo "6.pci_write_config_word" | tee -a $ERRORLOG_FILE
        echo "  "
		if [ -f "$OUTPUT_FILE" ];then
			awk '/error/ { for (i=2; i<=NF; i++) printf "%s ", $i; printf "\n"; }' $OUTPUT_FILE | tee -a $ERRORLOG_FILE
		fi
	fi
fi
create_make_file pciTest3.o

if [ ! -f "$MAKEFILE" ] ;then
	echo "Makefile creation error"
	exit 1
else
	OUTPUT_FILE=".output.txt"
	make -f $MAKEFILE &> $OUTPUT_FILE
	make -f $MAKEFILE >> $OUTPUT 2>&1
	if [ ! -f pciTest3.o ];then
		echo " " | tee -a $ERRORLOG_FILE
		echo "Please check the following PCI related functions. Some of the APIs are different." | tee -a $ERRORLOG_FILE
		echo "1.pci_assign_resource" | tee -a $ERRORLOG_FILE
		echo "2.pci_dev_driver" | tee -a $ERRORLOG_FILE
		echo "3.pci_resource_start" | tee -a $ERRORLOG_FILE
		echo "4.pci_set_drvdata" | tee -a $ERRORLOG_FILE
		echo "  " | tee -a $ERRORLOG_FILE
		if [ -f "$OUTPUT_FILE" ];then
			awk '/error/ { for (i=2; i<=NF; i++) printf "%s ", $i; printf "\n"; }' $OUTPUT_FILE | tee -a $ERRORLOG_FILE
		fi
	fi
fi

create_make_file user_accessTest.o

if [ ! -f "$MAKEFILE" ] ;then
	echo "Makefile creation error"
	exit 1
else
	OUTPUT_FILE=".output.txt"
	make -f $MAKEFILE &> $OUTPUT_FILE
	make -f $MAKEFILE >> $OUTPUT 2>&1
	
	if [ ! -f user_accessTest.o ];then
		echo " " | tee -a $ERRORLOG_FILE
		echo "Please check the following User Access related functions. Some of the APIs are different." | tee -a $ERRORLOG_FILE
		echo "1.copy_from_user" | tee -a $ERRORLOG_FILE
		echo "2.copy_to_user" | tee -a $ERRORLOG_FILE
        echo "  "
		if [ -f "$OUTPUT_FILE" ];then
			awk '/error/ { for (i=2; i<=NF; i++) printf "%s ", $i; printf "\n"; }' $OUTPUT_FILE | tee -a $ERRORLOG_FILE
		fi
	fi
fi

create_make_file memory_mgtTest1.o

if [ ! -f "$MAKEFILE" ] ;then
	echo "Makefile creation error"
	exit 1
else
	OUTPUT_FILE=".output.txt"
	make -f $MAKEFILE &> $OUTPUT_FILE
	make -f $MAKEFILE >> $OUTPUT 2>&1
	
	if [ ! -f memory_mgtTest1.o ];then
		echo " " | tee -a $ERRORLOG_FILE
		echo "Please check the following Memory Management related functions. Some of the APIs are different." | tee -a $ERRORLOG_FILE
		echo "1.alloc_pages" | tee -a $ERRORLOG_FILE
		echo "2.free_pages" | tee -a $ERRORLOG_FILE
		echo "3.__get_free_pages" | tee -a $ERRORLOG_FILE
		echo "4.kfree" | tee -a $ERRORLOG_FILE
		echo "5.kmalloc" | tee -a $ERRORLOG_FILE
		echo "6.vfree" | tee -a $ERRORLOG_FILE
        echo "  "
		if [ -f "$OUTPUT_FILE" ];then
			awk '/error/ { for (i=2; i<=NF; i++) printf "%s ", $i; printf "\n"; }' $OUTPUT_FILE | tee -a $ERRORLOG_FILE
		fi
	fi
fi

create_make_file memory_mgtTest2.o

if [ ! -f "$MAKEFILE" ] ;then
	echo "Makefile creation error"
	exit 1
else
	OUTPUT_FILE=".output.txt"
	make -f $MAKEFILE &> $OUTPUT_FILE
	make -f $MAKEFILE >> $OUTPUT 2>&1
	
	if [ ! -f memory_mgtTest2.o ];then
		echo " " | tee -a $ERRORLOG_FILE
		echo "Please check the following Memory Management related functions. Some of the APIs are different." | tee -a $ERRORLOG_FILE
		echo "1.do_mmap" | tee -a $ERRORLOG_FILE
		echo "2.do_munmap" | tee -a $ERRORLOG_FILE
		echo "3.get_page" | tee -a $ERRORLOG_FILE
		echo "4.iounmap"| tee -a $ERRORLOG_FILE
		echo "5.ioremap" | tee -a $ERRORLOG_FILE
		echo "6.memset" | tee -a $ERRORLOG_FILE
		echo "7.page_mapcount" | tee -a $ERRORLOG_FILE
		echo "8.put_page" | tee -a $ERRORLOG_FILE
        echo "  " | tee -a $ERRORLOG_FILE
		if [ -f "$OUTPUT_FILE" ];then
			awk '/error/ { for (i=2; i<=NF; i++) printf "%s ", $i; printf "\n"; }' $OUTPUT_FILE | tee -a $ERRORLOG_FILE
		fi
	fi
fi

create_make_file io_functionsTest.o

if [ ! -f "$MAKEFILE" ] ;then
	echo "Makefile creation error"
	exit 1
else
	OUTPUT_FILE=".output.txt"
	make -f $MAKEFILE &> $OUTPUT_FILE
	make -f $MAKEFILE >> $OUTPUT 2>&1
	
	if [ ! -f io_functionsTest.o ];then
		echo " " | tee -a $ERRORLOG_FILE
		echo "Please check the following I/O related functions. Some of the APIs are different." | tee -a $ERRORLOG_FILE
		echo "1.ioread32" | tee -a $ERRORLOG_FILE
		echo "2.iowrite32" | tee -a $ERRORLOG_FILE
		echo "3.printk" | tee -a $ERRORLOG_FILE
		echo "4.readb" | tee -a $ERRORLOG_FILE
		echo "5.readl" | tee -a $ERRORLOG_FILE
		echo "6.writel" | tee -a $ERRORLOG_FILE
        echo "  "
		if [ -f "$OUTPUT_FILE" ];then
			awk '/error/ { for (i=2; i<=NF; i++) printf "%s ", $i; printf "\n"; }' $OUTPUT_FILE | tee -a $ERRORLOG_FILE
		fi
	fi
fi

create_make_file synchronizationTest1.o

if [ ! -f "$MAKEFILE" ] ;then
	echo "Makefile creation error"
	exit 1
else
	OUTPUT_FILE=".output.txt"
	make -f $MAKEFILE &> $OUTPUT_FILE
	make -f $MAKEFILE >> $OUTPUT 2>&1
	
	if [ ! -f synchronizationTest1.o ];then
		echo " " | tee -a $ERRORLOG_FILE
		echo "Please check the following Synchronization related functions. Some of the APIs are different." | tee -a $ERRORLOG_FILE
		echo "1.atomic_add_negative" | tee -a $ERRORLOG_FILE
		echo "2.atomic_dec" | tee -a $ERRORLOG_FILE
		echo "3.atomic_inc" | tee -a $ERRORLOG_FILE
		echo "4.lock_kernel" | tee -a $ERRORLOG_FILE
		echo "5.mutex_lock" | tee -a $ERRORLOG_FILE
		echo "6.mutex_unlock" | tee -a $ERRORLOG_FILE
		echo "7.set_bit" | tee -a $ERRORLOG_FILE
		echo "8.unlock_kernel" | tee -a $ERRORLOG_FILE
        echo "  "
		if [ -f "$OUTPUT_FILE" ];then
			awk '/error/ { for (i=2; i<=NF; i++) printf "%s ", $i; printf "\n"; }' $OUTPUT_FILE | tee -a $ERRORLOG_FILE
		fi
	fi
fi

create_make_file synchronizationTest2.o

if [ ! -f "$MAKEFILE" ] ;then
	echo "Makefile creation error"
	exit 1
else
	OUTPUT_FILE=".output.txt"
	make -f $MAKEFILE &> $OUTPUT_FILE
	make -f $MAKEFILE >> $OUTPUT 2>&1
	
	if [ ! -f synchronizationTest2.o ];then
		echo " " | tee -a $ERRORLOG_FILE
		echo "Please check the following Synchronization related functions. Some of the APIs are different." | tee -a $ERRORLOG_FILE
		echo "1.down_write" | tee -a $ERRORLOG_FILE
		echo "2.spin_lock" | tee -a $ERRORLOG_FILE
		echo "3.spin_lock_init" | tee -a $ERRORLOG_FILE
		echo "4.spin_lock_irqsave" | tee -a $ERRORLOG_FILE
		echo "5.spin_unlock" | tee -a $ERRORLOG_FILE
		echo "6.spin_unlock_irqrestore" | tee -a $ERRORLOG_FILE
		echo "7.up_write" | tee -a $ERRORLOG_FILE
        echo "  "
		if [ -f "$OUTPUT_FILE" ];then
			awk '/error/ { for (i=2; i<=NF; i++) printf "%s ", $i; printf "\n"; }' $OUTPUT_FILE | tee -a $ERRORLOG_FILE
		fi
	fi
fi

create_make_file cpu_cache_Test.o

if [ ! -f "$MAKEFILE" ] ;then
	echo "Makefile creation error"
	exit 1
else
	OUTPUT_FILE=".output.txt"
	make -f $MAKEFILE &> $OUTPUT_FILE
	make -f $MAKEFILE >> $OUTPUT 2>&1
	
	if [ ! -f cpu_cache_Test.o ];then
		echo " " | tee -a $ERRORLOG_FILE
		echo "Please check the following CPU/Cache related functions. Some of the APIs are different." | tee -a $ERRORLOG_FILE
		echo "1.boot_cpu_has" | tee -a $ERRORLOG_FILE
		echo "2.flush_tlb_all" | tee -a $ERRORLOG_FILE
		echo "3.on_each_cpu" | tee -a $ERRORLOG_FILE
		echo "4.rdmsr" | tee -a $ERRORLOG_FILE
		echo "5.virt_to_phys" | tee -a $ERRORLOG_FILE
		echo "6.wbinvd" | tee -a $ERRORLOG_FILE
		echo "7.wmb" | tee -a $ERRORLOG_FILE
		echo "8.wrmsr" | tee -a $ERRORLOG_FILE
        echo "  "
		if [ -f "$OUTPUT_FILE" ];then
			awk '/error/ { for (i=2; i<=NF; i++) printf "%s ", $i; printf "\n"; }' $OUTPUT_FILE | tee -a $ERRORLOG_FILE
		fi
	fi
fi

create_make_file linkedlistTest.o

if [ ! -f "$MAKEFILE" ] ;then
	echo "Makefile creation error"
	exit 1
else
	OUTPUT_FILE=".output.txt"
	make -f $MAKEFILE &> $OUTPUT_FILE
	make -f $MAKEFILE >> $OUTPUT 2>&1
	
	if [ ! -f linkedlistTest.o ];then
		echo " " | tee -a $ERRORLOG_FILE
		echo "Please check the following Linked List related functions. Some of the APIs are different." | tee -a $ERRORLOG_FILE
		echo "1.list_add" | tee -a $ERRORLOG_FILE
		echo "2.list_del" | tee -a $ERRORLOG_FILE
		echo "3.list_for_each" | tee -a $ERRORLOG_FILE
        echo "  " | tee -a $ERRORLOG_FILE
		if [ -f "$OUTPUT_FILE" ];then
			awk '/error/ { for (i=2; i<=NF; i++) printf "%s ", $i; printf "\n"; }' $OUTPUT_FILE | tee -a $ERRORLOG_FILE
		fi
	fi
fi

create_make_file pageTest1.o

if [ ! -f "$MAKEFILE" ] ;then
	echo "Makefile creation error"
	exit 1
else
	OUTPUT_FILE=".output.txt"
	make -f $MAKEFILE &> $OUTPUT_FILE
	make -f $MAKEFILE >> $OUTPUT 2>&1
	if [ ! -f pageTest1.o ];then
		echo " " | tee -a $ERRORLOG_FILE
		echo "Please check the following Page related functions. Some of the APIs are different." | tee -a $ERRORLOG_FILE
		echo "1.change_page_attr" | tee -a $ERRORLOG_FILE
		echo "2.ClearPageReserved" | tee -a $ERRORLOG_FILE
		echo "3.copy_page" | tee -a $ERRORLOG_FILE
		echo "4.io_remap_pfn_range" | tee -a $ERRORLOG_FILE
		echo "5.SetPageLocked" | tee -a $ERRORLOG_FILE
		echo "6.SetPageReserved" | tee -a $ERRORLOG_FILE
		echo "7.unlock_page" | tee -a $ERRORLOG_FILE
		echo "8.virt_to_page" | tee -a $ERRORLOG_FILE
        echo "  "
		if [ -f "$OUTPUT_FILE" ];then
			awk '/error/ { for (i=2; i<=NF; i++) printf "%s ", $i; printf "\n"; }' $OUTPUT_FILE | tee -a $ERRORLOG_FILE
		fi
	fi
fi

create_make_file pageTest2.o

if [ ! -f "$MAKEFILE" ] ;then
	echo "Makefile creation error"
	exit 1
else
	OUTPUT_FILE=".output.txt"
	make -f $MAKEFILE &> $OUTPUT_FILE
	make -f $MAKEFILE >> $OUTPUT 2>&1
	
	if [ ! -f pageTest2.o ];then
		echo " " | tee -a $ERRORLOG_FILE
		echo "Please check the following Page related functions. Some of the APIs are different." | tee -a $ERRORLOG_FILE
		echo "1.pgd_offset" | tee -a $ERRORLOG_FILE
		echo "2.pmd_none" | tee -a $ERRORLOG_FILE
		echo "3.pmd_offset" | tee -a $ERRORLOG_FILE
		echo "4.pte_clear" | tee -a $ERRORLOG_FILE
		echo "5.pte_none" | tee -a $ERRORLOG_FILE
		echo "6.pte_page" | tee -a $ERRORLOG_FILE
		echo "7.pte_offset_map" | tee -a $ERRORLOG_FILE
		echo "8.pte_unmap" | tee -a $ERRORLOG_FILE
		echo "9.pud_none" | tee -a $ERRORLOG_FILE
        echo "  "
		if [ -f "$OUTPUT_FILE" ];then
			awk '/error/ { for (i=2; i<=NF; i++) printf "%s ", $i; printf "\n"; }' $OUTPUT_FILE | tee -a $ERRORLOG_FILE
		fi
	fi
fi

